@@ -1,23 +1,19 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-
-# Copyright (c) 2014-2020 Pytroll
-
-# Author(s):
-
-#   Adam.Dybbroe <adam.dybbroe@smhi.se>
-#   Panu Lahtinen <panu.lahtinen@fmi.fi>
-
+#
+# Copyright (c) 2014-2022 Pytroll developers
+#
+#
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
@@ -25,126 +21,15 @@
 
 import os
 import logging
-import tempfile
 import numpy as np
 from pyspectral.config import get_config
+from pyspectral.bandnames import BANDNAMES
 
 LOG = logging.getLogger(__name__)
 
 WAVE_LENGTH = 'wavelength'
 WAVE_NUMBER = 'wavenumber'
 
-BANDNAMES = {}
-BANDNAMES['generic'] = {'VIS006': 'VIS0.6',
-                        'VIS008': 'VIS0.8',
-                        'IR_016': 'NIR1.6',
-                        'IR_039': 'IR3.9',
-                        'WV_062': 'IR6.2',
-                        'WV_073': 'IR7.3',
-                        'IR_087': 'IR8.7',
-                        'IR_097': 'IR9.7',
-                        'IR_108': 'IR10.8',
-                        'IR_120': 'IR12.0',
-                        'IR_134': 'IR13.4',
-                        'HRV': 'HRV',
-                        'I01': 'I1',
-                        'I02': 'I2',
-                        'I03': 'I3',
-                        'I04': 'I4',
-                        'I05': 'I5',
-                        'M01': 'M1',
-                        'M02': 'M2',
-                        'M03': 'M3',
-                        'M04': 'M4',
-                        'M05': 'M5',
-                        'M06': 'M6',
-                        'M07': 'M7',
-                        'M08': 'M8',
-                        'M09': 'M9',
-                        'C01': 'ch1',
-                        'C02': 'ch2',
-                        'C03': 'ch3',
-                        'C04': 'ch4',
-                        'C05': 'ch5',
-                        'C06': 'ch6',
-                        'C07': 'ch7',
-                        'C08': 'ch8',
-                        'C09': 'ch9',
-                        'C10': 'ch10',
-                        'C11': 'ch11',
-                        'C12': 'ch12',
-                        'C13': 'ch13',
-                        'C14': 'ch14',
-                        'C15': 'ch15',
-                        'C16': 'ch16',
-                        }
-# handle arbitrary channel numbers
-for chan_num in range(1, 37):
-    BANDNAMES['generic'][str(chan_num)] = 'ch{:d}'.format(chan_num)
-
-# MODIS RSR files were made before 'chX' became standard in pyspectral
-BANDNAMES['modis'] = {str(chan_num): str(chan_num) for chan_num in range(1, 37)}
-
-BANDNAMES['avhrr-3'] = {'1': 'ch1',
-                        '2': 'ch2',
-                        '3b': 'ch3b',
-                        '3a': 'ch3a',
-                        '4': 'ch4',
-                        '5': 'ch5'}
-
-BANDNAMES['ahi'] = {'B01': 'ch1',
-                    'B02': 'ch2',
-                    'B03': 'ch3',
-                    'B04': 'ch4',
-                    'B05': 'ch5',
-                    'B06': 'ch6',
-                    'B07': 'ch7',
-                    'B08': 'ch8',
-                    'B09': 'ch9',
-                    'B10': 'ch10',
-                    'B11': 'ch11',
-                    'B12': 'ch12',
-                    'B13': 'ch13',
-                    'B14': 'ch14',
-                    'B15': 'ch15',
-                    'B16': 'ch16'
-                    }
-
-BANDNAMES['ami'] = {'VI004': 'ch1',
-                    'VI005': 'ch2',
-                    'VI006': 'ch3',
-                    'VI008': 'ch4',
-                    'NR013': 'ch5',
-                    'NR016': 'ch6',
-                    'SW038': 'ch7',
-                    'WV063': 'ch8',
-                    'WV069': 'ch9',
-                    'WV073': 'ch10',
-                    'IR087': 'ch11',
-                    'IR096': 'ch12',
-                    'IR105': 'ch13',
-                    'IR112': 'ch14',
-                    'IR123': 'ch15',
-                    'IR133': 'ch16'
-                    }
-
-BANDNAMES['fci'] = {'vis_04': 'ch1',
-                    'vis_05': 'ch2',
-                    'vis_06': 'ch3',
-                    'vis_08': 'ch4',
-                    'vis_09': 'ch5',
-                    'nir_13': 'ch6',
-                    'nir_16': 'ch7',
-                    'nir_22': 'ch8',
-                    'ir_38': 'ch9',
-                    'wv_63': 'ch10',
-                    'wv_73': 'ch11',
-                    'ir_87': 'ch12',
-                    'ir_97': 'ch13',
-                    'ir_105': 'ch14',
-                    'ir_123': 'ch15',
-                    'ir_133': 'ch16'
-                    }
 
 INSTRUMENTS = {'NOAA-19': 'avhrr/3',
                'NOAA-18': 'avhrr/3',
@@ -165,21 +50,25 @@ INSTRUMENTS = {'NOAA-19': 'avhrr/3',
                'Metop-C': 'avhrr/3',
                'Suomi-NPP': 'viirs',
                'NOAA-20': 'viirs',
+               'EOS-Aqua': 'modis',
+               'EOS-Terra': 'modis',
                'FY-3D': 'mersi-2',
                'FY-3C': 'virr',
                'FY-3B': 'virr',
                'Feng-Yun 3D': 'mersi-2',
+               'Meteosat-11': 'seviri',
+               'Meteosat-10': 'seviri',
+               'Meteosat-9': 'seviri',
+               'Meteosat-8': 'seviri',
                'FY-4A': 'agri',
                'GEO-KOMPSAT-2A': 'ami',
                'MTG-I1': 'fci'
                }
 
-
-HTTP_PYSPECTRAL_RSR = "https://zenodo.org/record/3824535/files/pyspectral_rsr_data.tgz"
-
+HTTP_PYSPECTRAL_RSR = "https://zenodo.org/record/6026563/files/pyspectral_rsr_data.tgz"
 
 RSR_DATA_VERSION_FILENAME = "PYSPECTRAL_RSR_VERSION"
-RSR_DATA_VERSION = "v1.0.15"
+RSR_DATA_VERSION = "v1.0.18"
 
 ATM_CORRECTION_LUT_VERSION = {}
 ATM_CORRECTION_LUT_VERSION['antarctic_aerosol'] = {'version': 'v1.0.1',
@@ -191,7 +80,7 @@ ATM_CORRECTION_LUT_VERSION['continental_clean_aerosol'] = {'version': 'v1.0.1',
 ATM_CORRECTION_LUT_VERSION['continental_polluted_aerosol'] = {'version': 'v1.0.1',
                                                               'filename': 'PYSPECTRAL_ATM_CORR_LUT_CPA'}
 ATM_CORRECTION_LUT_VERSION['desert_aerosol'] = {'version': 'v1.0.1',
-                                                           'filename': 'PYSPECTRAL_ATM_CORR_LUT_DA'}
+                                                'filename': 'PYSPECTRAL_ATM_CORR_LUT_DA'}
 ATM_CORRECTION_LUT_VERSION['marine_clean_aerosol'] = {'version': 'v1.0.1',
                                                       'filename': 'PYSPECTRAL_ATM_CORR_LUT_MCA'}
 ATM_CORRECTION_LUT_VERSION['marine_polluted_aerosol'] = {'version': 'v1.0.1',
@@ -205,7 +94,6 @@ ATM_CORRECTION_LUT_VERSION['urban_aerosol'] = {'version': 'v1.0.1',
 ATM_CORRECTION_LUT_VERSION['rayleigh_only'] = {'version': 'v1.0.1',
                                                'filename': 'PYSPECTRAL_ATM_CORR_LUT_RO'}
 
-
 AEROSOL_TYPES = ['antarctic_aerosol', 'continental_average_aerosol',
                  'continental_clean_aerosol', 'continental_polluted_aerosol',
                  'desert_aerosol', 'marine_clean_aerosol',
@@ -216,7 +104,6 @@ ATMOSPHERES = {'subarctic summer': 4, 'subarctic winter': 5,
                'midlatitude summer': 6, 'midlatitude winter': 7,
                'tropical': 8, 'us-standard': 9}
 
-
 HTTPS_RAYLEIGH_LUTS = {}
 URL_PREFIX = "https://zenodo.org/record/1288441/files/pyspectral_atm_correction_luts"
 for atype in AEROSOL_TYPES:
@@ -225,22 +112,11 @@ for atype in AEROSOL_TYPES:
     HTTPS_RAYLEIGH_LUTS[atype] = url
 
 
-CONF = get_config()
-LOCAL_RSR_DIR = CONF.get('rsr_dir')
-LOCAL_RAYLEIGH_DIR = CONF.get('rayleigh_dir')
-
-try:
-    os.makedirs(LOCAL_RSR_DIR)
-except OSError:
-    if not os.path.isdir(LOCAL_RSR_DIR):
-        raise
-
-RAYLEIGH_LUT_DIRS = {}
-for sub_dir_name in HTTPS_RAYLEIGH_LUTS:
-    dirname = os.path.join(LOCAL_RAYLEIGH_DIR, sub_dir_name)
-    RAYLEIGH_LUT_DIRS[sub_dir_name] = dirname
-
-TB2RAD_DIR = CONF.get('tb2rad_dir', tempfile.gettempdir())
+def get_rayleigh_lut_dir(aerosol_type):
+    """Get the rayleight LUT directory for the specified aerosol type."""
+    conf = get_config()
+    local_rayleigh_dir = conf.get('rayleigh_dir')
+    return os.path.join(local_rayleigh_dir, aerosol_type)
 
 
 def convert2wavenumber(rsr):
@@ -274,7 +150,7 @@ def convert2wavenumber(rsr):
                     retv[chname][det]['wavenumber'] = wnum[::-1]
                 elif sat == "response":
                     # Flip the response array:
-                    if type(rsr[chname][det][sat]) is dict:
+                    if isinstance(rsr[chname][det][sat], dict):
                         retv[chname][det][sat] = {}
                         for name in rsr[chname][det][sat].keys():
                             resp = rsr[chname][det][sat][name]
@@ -324,15 +200,13 @@ def get_bandname_from_wavelength(sensor, wavelength, rsr, epsilon=0.1, multiple_
 
     if len(chfound) == 1:
         return chfound[0]
-    elif len(chfound) > 1:
+    if len(chfound) > 1:
         bstrlist = ['band={}'.format(b) for b in chfound]
         if not multiple_bands:
             raise AttributeError("More than one band found with that wavelength! {}".format(str(bstrlist)))
-        else:
-            LOG.debug("More than one band found with requested wavelength: %s", str(bstrlist))
+        LOG.debug("More than one band found with requested wavelength: %s", str(bstrlist))
         return chfound
-    else:
-        return None
+    return None
 
 
 def sort_data(x_vals, y_vals):
@@ -407,7 +281,9 @@ def download_rsr(**kwargs):
     except ImportError:
         TQDM_LOADED = False
 
-    dest_dir = kwargs.get('dest_dir', LOCAL_RSR_DIR)
+    config = get_config()
+    local_rsr_dir = config.get('rsr_dir')
+    dest_dir = kwargs.get('dest_dir', local_rsr_dir)
     dry_run = kwargs.get('dry_run', False)
 
     LOG.info("Download RSR files and store in directory %s", dest_dir)
@@ -454,15 +330,14 @@ def download_luts(**kwargs):
     else:
         aerosol_types = HTTPS_RAYLEIGH_LUTS.keys()
 
-    chunk_size = 1024*1024  # 1 MB
-
+    chunk_size = 1024 * 1024  # 1 MB
     for subname in aerosol_types:
 
         LOG.debug('Aerosol type: %s', subname)
         http = HTTPS_RAYLEIGH_LUTS[subname]
         LOG.debug('URL = %s', http)
 
-        subdir_path = RAYLEIGH_LUT_DIRS[subname]
+        subdir_path = get_rayleigh_lut_dir(subname)
         try:
             LOG.debug('Create directory: %s', subdir_path)
             if not dry_run:
@@ -527,7 +402,6 @@ class NullHandler(logging.Handler):
 
     def emit(self, record):
         """Record a message."""
-        pass
 
 
 def logging_off():
@@ -541,3 +415,69 @@ def get_logger(name):
     if not log.handlers:
         log.addHandler(NullHandler())
     return log
+
+
+def get_wave_range(in_chan, threshold=0.15):
+    """Return central, min and max wavelength in an RSR greater than threshold.
+
+    An RSR function will generally start near zero, increase to a maximum and
+    then drop back to near zero. This function takes advantage of this to find
+    the first and last points where the RSR is greater than a threshold. These
+    points are then defined as the minimum and maximum wavelengths for a
+    given channel, and can be used, for example, in Satpy reader YAML files.
+
+    """
+    cwl = get_central_wave(in_chan['wavelength'], in_chan['response'])
+
+    wvls = in_chan['wavelength']
+    rsr = in_chan['response']
+
+    pts = (rsr > threshold).nonzero()
+    min_wvl = wvls[pts[0][0]]
+    max_wvl = wvls[pts[0][-1]]
+
+    return [min_wvl, cwl, max_wvl]
+
+
+def convert2str(value):
+    """Convert a value to string.
+
+    Args:
+        value: Either a str, bytes or 1-element numpy array
+
+    """
+    value = bytes2string(value)
+    return np2str(value)
+
+
+def np2str(value):
+    """Convert an `numpy.string_` to str.
+
+    Args:
+        value (ndarray): scalar or 1-element numpy array to convert
+    Raises:
+        ValueError: if value is array larger than 1-element or it is not of
+                    type `numpy.string_` or it is not a numpy array
+
+    """
+    if isinstance(value, str):
+        return value
+
+    if hasattr(value, 'dtype') and \
+            issubclass(value.dtype.type, (np.str_, np.string_, np.object_)) \
+            and value.size == 1:
+        value = value.item()
+        # python 3 - was scalar numpy array of bytes
+        # otherwise python 2 - scalar numpy array of 'str'
+        if not isinstance(value, str):
+            return value.decode()
+        return value
+
+    raise ValueError("Array is not a string type or is larger than 1")
+
+
+def bytes2string(var):
+    """Decode a bytes variable and return a string."""
+    if isinstance(var, bytes):
+        return var.decode('utf-8')
+    return var
